SHELL=bash
DATAPATH = ../data/$(DATA)
CORRECTRESULT = ../data/$(DATA)/correctout.txt

PROGRAM ?= $(INPUT)

SIMPARAMS ?= -cycle -conf fpga

TARFILE = mergesort_sln.tgz

help:
	@echo "USAGE: "
	@echo "  make help    : Displays this information."
	@echo "  make clean   : Removes all files created during a compilation or a run."
	@echo "  make compile : Compiles the input file with the given data set."
	@echo "  make run     : Compiles and runs the input file with the given data set."
	@echo "  make check   : Compiles, runs and compares the output against the correct"
	@echo "                 output."
	@echo "  make testall : Checks the correctness of merge.s.c, merge.p.c and mergesort.c"
	@echo "                 against all the data sets. Creates a testlog.txt file. Search"
	@echo "                 this file for errors in order to see the cases that fail. Note"
	@echo "                 that this command will not quit on error, it will continue until"
	@echo "                 it runs all the cases."
	@echo "  make crossover : Runs mergesort.c with data set dms3 against values of the"
	@echo "                   crossover parameter from 2 to 18. The resulting cycle counts"
	@echo "                   are reported in cycle.txt file."
	@echo "  make submit  : Packs assignment package for grading. The input will be"
	@echo "                 compressed into mergesort_sln.tgz."
	@echo "  make submitcheck : Checks if all the deliverables for submission are present."
	@echo ""
	@echo "  Compile, run and check commands require setting the INPUT (or PROGRAM) and DATA"
	@echo "  arguments on command line."
	@echo "  Following are some examples:"
	@echo ""
	@echo "    make check INPUT=merge.s.c DATA=dm1"
	@echo "    make run INPUT=mergesort.c DATA=dms3"
	@echo "    make testall"
	@echo "    make crossover"

check: compile
	@echo "xmtsim $(SIMPARAMS) $(patsubst %.c,%,$(PROGRAM)).sim -binload $(patsubst %.c,%,$(PROGRAM)).b -textdump dump -dumpvar R"
	@xmtsim $(SIMPARAMS) $(patsubst %.c,%,$(PROGRAM)).sim -binload $(patsubst %.c,%,$(PROGRAM)).b -textdump dump -dumpvar R; \
	 [[ $$? != 0 ]] && echo "ERROR: Run on the simulator failed!" && exit 1 || exit 0
	cat dump.R | sed 's/[[:space:]][[:space:]]*/\n/g'|egrep -v "^$$">tmp.txt
	mv tmp.txt dump.R
	@echo "diff dump.R $(CORRECTRESULT) >/dev/null"
	@diff dump.R $(CORRECTRESULT) >/dev/null; \
	 [[ $$? != 0 ]] && echo "ERROR: dump.R does not match $(CORRECTRESULT)!" \
		&& exit 1 || exit 0
	@echo "Your program passed the correctness test!"

run: compile
	@echo "xmtsim $(SIMPARAMS) $(patsubst %.c,%,$(PROGRAM)).sim -binload $(patsubst %.c,%,$(PROGRAM)).b -textdump dump -dumpvar R"
	@xmtsim $(SIMPARAMS) $(patsubst %.c,%,$(PROGRAM)).sim -binload $(patsubst %.c,%,$(PROGRAM)).b -textdump dump -dumpvar R; \
	 [[ $$? != 0 ]] && echo "ERROR: Run on the simulator failed!" && exit 1 || exit 0


compile: checkparams
	@echo "xmtcc $(COMPILER_PARAMS) -include $(DATAPATH)/*.h $(DATAPATH)/*.xbo $(PROGRAM) -o $(patsubst %.c,%,$(PROGRAM))"
	@xmtcc $(COMPILER_PARAMS) -include $(DATAPATH)/*.h $(DATAPATH)/*.xbo $(PROGRAM) \
		-o $(patsubst %.c,%,$(PROGRAM)); \
	 [[ $$? != 0 ]] && echo "ERROR: Compilation failed!" && exit 1 || exit 0

checkparams:
	@if [[ "$(PROGRAM)" == "" ]]; then \
		echo "ERROR: PROGRAM variable is not set."; \
		exit 1; \
	fi
	@if ! [[ -e $(PROGRAM) ]]; then \
		echo "ERROR: $(PROGRAM) does not exist."; \
		exit 1; \
	fi
	@if [[ "$(DATA)" == "" ]]; then \
		echo "ERROR: DATA variable is not set."; \
		exit 1; \
	fi
	@if ! [[ -e $(DATAPATH) ]]; then \
		echo "ERROR: Data $(DATA) does not exist."; \
		exit 1; \
	fi

testall:
	$(MAKE) subtestall 2>&1| tee testlog.txt

crossover:
	$(MAKE) subcrossover 2>&1| tee log.txt
	cat log.txt |grep "Total Execution Time = " |sed 's/Total Execution Time = //g'>cycle.txt
	rm -f log.txt

subcrossover:
	$(MAKE) check COMPILER_PARAMS="-quiet -D CROSSOVER=2" PROGRAM=mergesort.c DATA=dms3
	$(MAKE) check COMPILER_PARAMS="-quiet -D CROSSOVER=3" PROGRAM=mergesort.c DATA=dms3
	$(MAKE) check COMPILER_PARAMS="-quiet -D CROSSOVER=4" PROGRAM=mergesort.c DATA=dms3
	$(MAKE) check COMPILER_PARAMS="-quiet -D CROSSOVER=5" PROGRAM=mergesort.c DATA=dms3
	$(MAKE) check COMPILER_PARAMS="-quiet -D CROSSOVER=6" PROGRAM=mergesort.c DATA=dms3
	$(MAKE) check COMPILER_PARAMS="-quiet -D CROSSOVER=7" PROGRAM=mergesort.c DATA=dms3
	$(MAKE) check COMPILER_PARAMS="-quiet -D CROSSOVER=8" PROGRAM=mergesort.c DATA=dms3
	$(MAKE) check COMPILER_PARAMS="-quiet -D CROSSOVER=9" PROGRAM=mergesort.c DATA=dms3
	$(MAKE) check COMPILER_PARAMS="-quiet -D CROSSOVER=10" PROGRAM=mergesort.c DATA=dms3
	$(MAKE) check COMPILER_PARAMS="-quiet -D CROSSOVER=11" PROGRAM=mergesort.c DATA=dms3
	$(MAKE) check COMPILER_PARAMS="-quiet -D CROSSOVER=12" PROGRAM=mergesort.c DATA=dms3
	$(MAKE) check COMPILER_PARAMS="-quiet -D CROSSOVER=13" PROGRAM=mergesort.c DATA=dms3
	$(MAKE) check COMPILER_PARAMS="-quiet -D CROSSOVER=14" PROGRAM=mergesort.c DATA=dms3
	$(MAKE) check COMPILER_PARAMS="-quiet -D CROSSOVER=15" PROGRAM=mergesort.c DATA=dms3
	$(MAKE) check COMPILER_PARAMS="-quiet -D CROSSOVER=16" PROGRAM=mergesort.c DATA=dms3
	$(MAKE) check COMPILER_PARAMS="-quiet -D CROSSOVER=17" PROGRAM=mergesort.c DATA=dms3
	$(MAKE) check COMPILER_PARAMS="-quiet -D CROSSOVER=18" PROGRAM=mergesort.c DATA=dms3

subtestall:
	-$(MAKE) check COMPILER_PARAMS="-quiet -D CROSSOVER=4" PROGRAM=merge.p.c DATA=dm1 
	-$(MAKE) check COMPILER_PARAMS="-quiet -D CROSSOVER=4" PROGRAM=merge.p.c DATA=dm2
	-$(MAKE) check COMPILER_PARAMS="-quiet -D CROSSOVER=4" PROGRAM=merge.p.c DATA=dm3
	-$(MAKE) check COMPILER_PARAMS="-quiet -D CROSSOVER=4" PROGRAM=merge.s.c DATA=dm1
	-$(MAKE) check COMPILER_PARAMS="-quiet -D CROSSOVER=4" PROGRAM=merge.s.c DATA=dm2
	-$(MAKE) check COMPILER_PARAMS="-quiet -D CROSSOVER=4" PROGRAM=merge.s.c DATA=dm3
	-$(MAKE) check COMPILER_PARAMS="-quiet -D CROSSOVER=4" PROGRAM=mergesort.c DATA=dms1
	-$(MAKE) check COMPILER_PARAMS="-quiet -D CROSSOVER=4" PROGRAM=mergesort.c DATA=dms2
	-$(MAKE) check COMPILER_PARAMS="-quiet -D CROSSOVER=4" PROGRAM=mergesort.c DATA=dms3

submitcheck:
	@if ! [[ -e mergesort.c ]]; then echo "FAIL: mergesort.c not found."; else \
		echo "OK: Found mergesort.c"; fi
	@if ! [[ -e merge.s.c ]]; then echo "FAIL: merge.s.c not found."; else \
		echo "OK: Found merge.s.c"; fi
	@if ! [[ -e merge.p.c ]]; then echo "FAIL: merge.p.c not found."; else \
		echo "OK: Found merge.s.c"; fi
	@if ! [[ -e ../crossover.pdf ]]; then echo "FAIL: crossover.pdf not found."; else \
		echo "OK: Found crossover.pdf"; fi
	@if [[ -e mergesort.c ]]; then \
	    grep '#define CROSSOVER' mergesort.c|sed 's/.*CROSSOVER//g'| \
			awk '{if($$1 ~ /^[0-9][0-9]*$$/) print "OK: Reported cross-over value: " $$1; else print "FAIL: No cross-over value reported.";}'; \
	 else \
		echo "FAIL: No cross-over value reported."; \
	 fi; \

submit:
	@cd ..; \
	 tar czvf $(TARFILE) `find . -type f -print`; \
	 echo "Your assignment has been packaged as $(TARFILE)."

clean:
	rm -rf *.b *.sim dump.R *~ *.log
